#include <stdafx.h>
#include <dinput.h>
#include "resource.h"

#include "AboutBox.h"
#include "TestDialog.h"
#include "config.h"
#include "controller.h"
#include "Kaillera.h"
#include "di.h"

ControllerDef Controllers[4];
bool g_bUseKaillera = false;
bool g_bKailleraViewLog = false;
char g_szKailleraDll[256] = ".\\kailleraclient.dll";
int g_iKailleraPlayAs = 4;

#define NooTe_KEY "Software\\NooTe\\NooTe_Di"
#define KEY_Kaillera "Kaillera"
#define KEY_KailleraPlayAs "PlayAs"
#define KEY_KailleraDll "Dll"
#define KEY_KailleraViewLog "ViewLog"
#define KEY_Controller "Controller%d"
#define KEY_DIDevice "DIDevice"
#define KEY_StickRight "StickRight"
#define KEY_StickLeft "StickLeft"
#define KEY_StickUp "StickUp"
#define KEY_StickDown "StickDown"
#define KEY_StickLeftRight "StickLeftRight"
#define KEY_StickUpDown "StickUpDown"
#define KEY_TriggerLeft "TriggerLeft"
#define KEY_TriggerRight "TriggerRight"
#define KEY_TriggerZ "TriggerZ"
#define KEY_ButtonA "ButtonA"
#define KEY_ButtonB "ButtonB"
#define KEY_ButtonStart "ButtonStart"
#define KEY_ButtonUp "ButtonUp"
#define KEY_ButtonDown "ButtonDown"
#define KEY_ButtonLeft "ButtonLeft"
#define KEY_ButtonRight "ButtonRight"
#define KEY_PadUp "PadUp"
#define KEY_PadDown "PadDown"
#define KEY_PadLeft "PadLeft"
#define KEY_PadRight "PadRight"

#define LABEL_Disable "Disable"
#define LABEL_N64Controller1 "N64 Controller 1"
#define LABEL_N64Controller2 "N64 Controller 2"
#define LABEL_N64Controller3 "N64 Controller 3"
#define LABEL_N64Controller4 "N64 Controller 4"
#define LABEL_Spectator "Spectator"
#define LABEL_JoystickAxeX "Axe X"
#define LABEL_JoystickInversedAxeX "Inversed Axe X"
#define LABEL_JoystickAxeY "Axe Y"
#define LABEL_JoystickInversedAxeY "Inversed Axe Y"
#define LABEL_JoystickAxeZ "Axe Z"
#define LABEL_JoystickInversedAxeZ "Inversed Axe Z"
#define LABEL_UseButtons "Use Buttons"
#define LABEL_Unused "(unused)"

void SetVoidButtons(BUTTONS *pButtons)
{
	pButtons->button = 0;
	pButtons->X_AXIS = 0;
	pButtons->Y_AXIS = 0;
}

void CopyButtons(BUTTONS *pButtons1, BUTTONS *pButtons2)
{
	pButtons1->button = pButtons2->button;
	pButtons1->X_AXIS = pButtons2->X_AXIS;
	pButtons1->Y_AXIS = pButtons2->Y_AXIS;
}

void SetAsNoController(ControllerDef *controller)
{
	controller->diHandle = -1;
}

void SetAsKeyboardController(ControllerDef *controller, int diHandle)
{
  controller->diHandle = diHandle;
}

void SetAsDefaultKeyboardController(ControllerDef *controller)
{
  controller->ButtonA = DIK_LSHIFT;
  controller->ButtonB = DIK_LCONTROL;
  
  controller->ButtonStart = DIK_SPACE;

  controller->ButtonCUp = DIK_NUMPAD8;
  controller->ButtonCDown = DIK_NUMPAD2;
  controller->ButtonCLeft = DIK_NUMPAD4;
  controller->ButtonCRight = DIK_NUMPAD6;

  controller->PadUp = DIK_HOME;
  controller->PadDown = DIK_END;
  controller->PadLeft = DIK_DELETE;
  controller->PadRight = DIK_NEXT;

  controller->StickRight = DIK_RIGHT;
  controller->StickLeft = DIK_LEFT;
  controller->StickUp = DIK_UP;
  controller->StickDown = DIK_DOWN;

  controller->TriggerZ = DIK_Z;
  controller->TriggerLeft = DIK_A;
  controller->TriggerRight = DIK_S;
}

void SetAsJoystickController(ControllerDef *controller, int diHandle)
{
  controller->diHandle = diHandle;
}

void SetAsDefaultJoystickController(ControllerDef *controller, int diHandle)
{
  SetAsJoystickController(controller, diHandle);

  controller->ButtonA = 0;
  controller->ButtonB = 3;
  
  controller->ButtonStart = 8;

  controller->ButtonCUp = 5;
  controller->ButtonCDown = 1;
  controller->ButtonCLeft = 4;
  controller->ButtonCRight = 2;

  controller->PadUp = 10;
  controller->PadDown = 11;
  controller->PadLeft = 12;
  controller->PadRight = 13;

  controller->AxeForStickLeftRight = 0;
  controller->AxeForStickUpDown = 3;

  controller->StickLeft = 0;
  controller->StickRight = 0;
  controller->StickUp = 0;
  controller->StickDown = 0;

  controller->TriggerZ = 9;
  controller->TriggerLeft = 6;
  controller->TriggerRight = 7;
}

void SetDefaultController()
{
  SetAsNoController(&Controllers[0]);
  SetAsNoController(&Controllers[1]);
  SetAsNoController(&Controllers[2]);
  SetAsNoController(&Controllers[3]);
}

void ReadDevice(ControllerDef *controller, HKEY hKey)
{
  char szData[MAX_PATH];
  DWORD	rc;
  DWORD cbData, dwType;

  SetAsDefaultKeyboardController(controller);

#define MACRO_QueryValue(KEY, where) \
  cbData = sizeof(szData); \
  rc = RegQueryValueEx(	hKey, KEY, NULL, &dwType, (LPBYTE)szData, &cbData); \
  if (rc == ERROR_SUCCESS && cbData != 0) \
    where = atoi(szData);

  MACRO_QueryValue(KEY_DIDevice, controller->diHandle);

  MACRO_QueryValue(KEY_StickUpDown, controller->AxeForStickUpDown);
  MACRO_QueryValue(KEY_StickLeftRight, controller->AxeForStickLeftRight);

  MACRO_QueryValue(KEY_StickUp, controller->StickUp);
  MACRO_QueryValue(KEY_StickDown, controller->StickDown);
  MACRO_QueryValue(KEY_StickLeft, controller->StickLeft);
  MACRO_QueryValue(KEY_StickRight, controller->StickRight);

  MACRO_QueryValue(KEY_TriggerLeft, controller->TriggerLeft);
  MACRO_QueryValue(KEY_TriggerRight, controller->TriggerRight);
  MACRO_QueryValue(KEY_TriggerZ, controller->TriggerZ);

  MACRO_QueryValue(KEY_ButtonA, controller->ButtonA);
  MACRO_QueryValue(KEY_ButtonB, controller->ButtonB);
  MACRO_QueryValue(KEY_ButtonStart, controller->ButtonStart);
  
  MACRO_QueryValue(KEY_ButtonUp, controller->ButtonCUp);
  MACRO_QueryValue(KEY_ButtonDown, controller->ButtonCDown);
  MACRO_QueryValue(KEY_ButtonLeft, controller->ButtonCLeft);
  MACRO_QueryValue(KEY_ButtonRight, controller->ButtonCRight);

  MACRO_QueryValue(KEY_PadUp, controller->PadUp);
  MACRO_QueryValue(KEY_PadDown, controller->PadDown);
  MACRO_QueryValue(KEY_PadLeft, controller->PadLeft);
  MACRO_QueryValue(KEY_PadRight, controller->PadRight);

#undef MACRO_QueryValue
}

void WriteDevice(ControllerDef *controller, HKEY hKey)
{
  char temp[256];

#define MACRO_SetValue(KEY, num) \
  sprintf(temp, "%d", num); \
  RegSetValueEx(hKey, KEY, NULL, REG_SZ, (BYTE *)temp, sizeof(temp));

  MACRO_SetValue(KEY_DIDevice, controller->diHandle);

  MACRO_SetValue(KEY_StickUpDown, controller->AxeForStickUpDown);
  MACRO_SetValue(KEY_StickLeftRight, controller->AxeForStickLeftRight);

  MACRO_SetValue(KEY_TriggerLeft, controller->TriggerLeft);
  MACRO_SetValue(KEY_TriggerRight, controller->TriggerRight);
  MACRO_SetValue(KEY_TriggerZ, controller->TriggerZ);

  MACRO_SetValue(KEY_ButtonA, controller->ButtonA);
  MACRO_SetValue(KEY_ButtonB, controller->ButtonB);
  MACRO_SetValue(KEY_ButtonStart, controller->ButtonStart);
  
  MACRO_SetValue(KEY_ButtonUp, controller->ButtonCUp);
  MACRO_SetValue(KEY_ButtonDown, controller->ButtonCDown);
  MACRO_SetValue(KEY_ButtonLeft, controller->ButtonCLeft);
  MACRO_SetValue(KEY_ButtonRight, controller->ButtonCRight);

  MACRO_SetValue(KEY_PadUp, controller->PadUp);
  MACRO_SetValue(KEY_PadDown, controller->PadDown);
  MACRO_SetValue(KEY_PadLeft, controller->PadLeft);
  MACRO_SetValue(KEY_PadRight, controller->PadRight);

#undef MACRO_SetValue
}

void ReadKaillera(HKEY hKey)
{
  char szData[MAX_PATH];
  DWORD	rc;
  DWORD cbData, dwType;

	cbData = sizeof(szData);
  rc = RegQueryValueEx(	hKey, KEY_KailleraPlayAs, NULL, &dwType, (LPBYTE)szData, &cbData);
  if(rc == ERROR_SUCCESS)
		g_iKailleraPlayAs = atoi(szData);
	cbData = sizeof(szData);

	cbData = sizeof(szData);
  rc = RegQueryValueEx(	hKey, KEY_KailleraViewLog, NULL, &dwType, (LPBYTE)szData, &cbData);
  if(rc == ERROR_SUCCESS)
		g_bKailleraViewLog = atoi(szData)>0?true:false;
	cbData = sizeof(szData);

  rc = RegQueryValueEx(	hKey, KEY_KailleraDll, NULL, &dwType, (LPBYTE)szData, &cbData);
  if(rc == ERROR_SUCCESS)
		strncpy(g_szKailleraDll, szData, 255);
}

void WriteKaillera(HKEY hKey)
{
	char szTemp[256];

	sprintf(szTemp, "%d", g_iKailleraPlayAs);
  RegSetValueEx(hKey, KEY_KailleraPlayAs, NULL, REG_SZ, (BYTE *)szTemp, strlen(szTemp));

	sprintf(szTemp, "%d", g_bKailleraViewLog?1:0);
  RegSetValueEx(hKey, KEY_KailleraViewLog, NULL, REG_SZ, (BYTE *)szTemp, strlen(szTemp));
	
	RegSetValueEx(hKey, KEY_KailleraDll, NULL, REG_SZ, (BYTE *)g_szKailleraDll, strlen(g_szKailleraDll));
}

static void Complain(HWND hwndOwner,HRESULT hr,LPCSTR pszMessage)
{
  MessageBox(hwndOwner, pszMessage, "NooTe_Di Error!", MB_OK);
}

void ReadConfiguration()
{
  HKEY	hKey1, hKey2, hKey3;
  DWORD	rc;
  
  if (RegConnectRegistry(NULL,
    HKEY_CURRENT_USER, &hKey1) == ERROR_SUCCESS)
  {
    char	szBuffer[260];
    
    strcpy(szBuffer, NooTe_KEY);
    
    rc = RegOpenKeyEx(hKey1, szBuffer, 0, KEY_ALL_ACCESS, &hKey2);
    if(rc == ERROR_SUCCESS)
    {
			rc = RegOpenKeyEx(hKey2, KEY_Kaillera, 0, KEY_ALL_ACCESS, &hKey3);
      if(rc == ERROR_SUCCESS)
      {
				ReadKaillera(hKey3);
				RegCloseKey(hKey3);
			}

      for(int c=0; c<4; c++)
      {
        char	szController[260];
    
        sprintf(szController, KEY_Controller, c);

        rc = RegOpenKeyEx(hKey2, szController, 0, KEY_ALL_ACCESS, &hKey3);
        if(rc == ERROR_SUCCESS)
          ReadDevice(&Controllers[c], hKey3);
        else
					SetAsNoController(&Controllers[c]);
        RegCloseKey(hKey3);
      }
      RegCloseKey(hKey2);
    }
    RegCloseKey(hKey1);
  }
}
void WriteConfiguration()
{
  HKEY	hKey1, hKey2, hKey3;
  DWORD	rc;
  
  if (RegConnectRegistry(NULL, HKEY_CURRENT_USER, &hKey1) == ERROR_SUCCESS)
  {
    char	szBuffer[260];
    
    strcpy(szBuffer, NooTe_KEY);
    
    rc = RegOpenKeyEx(hKey1, szBuffer, 0, KEY_ALL_ACCESS, &hKey2);
    if(rc != ERROR_SUCCESS)
      rc = RegCreateKeyEx(hKey1, szBuffer, 0, NULL, 0, KEY_ALL_ACCESS, NULL, &hKey2, 0);
    if(rc == ERROR_SUCCESS)
    {
			rc = RegOpenKeyEx(hKey2, KEY_Kaillera, 0, KEY_ALL_ACCESS, &hKey3);
      if(rc != ERROR_SUCCESS)
        rc = RegCreateKeyEx(hKey2, KEY_Kaillera, 0, NULL, 0, KEY_ALL_ACCESS, NULL, &hKey3, 0);
			if(rc == ERROR_SUCCESS)
			{
				WriteKaillera(hKey3);
				RegCloseKey(hKey3);
			}

      for(int c=0; c<4; c++)
      {
        char	szController[260];
    
        sprintf(szController, KEY_Controller, c);

        rc = RegOpenKeyEx(hKey2, szController, 0, KEY_ALL_ACCESS, &hKey3);
        if(rc != ERROR_SUCCESS)
          rc = RegCreateKeyEx(hKey2, szController, 0, NULL, 0, KEY_ALL_ACCESS, NULL, &hKey3, 0);
        if(rc == ERROR_SUCCESS)
					WriteDevice(&Controllers[c], hKey3);

        RegCloseKey(hKey3);
      }
      RegCloseKey(hKey2);
    }
    RegCloseKey(hKey1);
  }
}

/////////////////////////////////////////////////////////////////////////////
// CConfigurationDlg dialog

class CConfigurationDlg : public CDialog
{
// Construction
public:
	CConfigurationDlg(CWnd* pParent = NULL);   // standard constructor

// Dialog Data
	//{{AFX_DATA(CConfigurationDlg)
	enum { IDD = IDD_CONFIGURATION };
	CButton	m_checkKailleraViewLog;
	CStatic	m_staticKailleraDll;
	CStatic	m_staticKailleraVersion;
	CStatic	m_staticKailleraPlayer;
	CComboBox	m_comboKailleraPlayer;
	CButton	m_checkKaillera;
	CButton	m_buttonSetting;
	CComboBox	m_comboDevice;
	CComboBox	m_comboControllers;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CConfigurationDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CConfigurationDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnButtonAbout();
	afx_msg void OnButtonTest();
  virtual void OnOK();
	afx_msg void OnSelendokComboController();
	afx_msg void OnSelendokComboDevice();
	afx_msg void OnButtonKeySetting();
	afx_msg void OnCheckKaillera();
	afx_msg void OnSelendokComboKailleraPlayer();
	afx_msg void OnCheckKailleraViewLog();
	afx_msg void OnButtonKailleraDll();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
// CJoystickSetting dialog

class CJoystickSetting : public CDialog
{
// Construction
public:
  CJoystickSetting(CWnd* pParent = NULL);   // standard constructor

// Dialog Data
	//{{AFX_DATA(CJoystickSetting)
	enum { IDD = IDD_DIALOG_JOYSTICK_SETTING };
	CComboBox	m_comboStickUpDown;
	CComboBox	m_comboStickLeftRight;
	//}}AFX_DATA

  ControllerDef *controller;
	
  void SetMapping();
  __int32 ButtonMapping(LONG idButton);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CJoystickSetting)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CJoystickSetting)
	virtual BOOL OnInitDialog();
	afx_msg void OnButtonDefault();
	afx_msg void OnButtonStart();
	afx_msg void OnButtonPadUp();
	afx_msg void OnButtonPadRight();
	afx_msg void OnButtonPadLeft();
	afx_msg void OnButtonPadDown();
	afx_msg void OnButtonCUp();
	afx_msg void OnButtonCRight();
	afx_msg void OnButtonCLeft();
	afx_msg void OnButtonCDown();
	afx_msg void OnButtonB();
	afx_msg void OnButtonA();
	afx_msg void OnButtonTriggerLeft();
	afx_msg void OnButtonTriggerRight();
	afx_msg void OnButtonTriggerZ();
	afx_msg void OnSelendokComboStickUpdown();
	afx_msg void OnSelendokComboStickLeftright();
	afx_msg void OnButtonStickDown();
	afx_msg void OnButtonStickLeft();
	afx_msg void OnButtonStickRight();
	afx_msg void OnButtonStickUp();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
// CKeyboardSetting dialog

class CKeyboardSetting : public CDialog
{
// Construction
public:
	CKeyboardSetting(CWnd* pParent = NULL);   // standard constructor

// Dialog Data
	//{{AFX_DATA(CKeyboardSetting)
	enum { IDD = IDD_DIALOG_KEYBOARD_SETTING };
	//}}AFX_DATA

  ControllerDef *controller;

  void SetMapping();
  __int32 KeyMapping(LONG idButton);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CKeyboardSetting)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CKeyboardSetting)
	virtual BOOL OnInitDialog();
	afx_msg void OnButtonDefault();
	afx_msg void OnButtonStart();
	afx_msg void OnButtonStickUp();
	afx_msg void OnButtonStickRight();
	afx_msg void OnButtonStickLeft();
	afx_msg void OnButtonStickDown();
	afx_msg void OnButtonPadUp();
	afx_msg void OnButtonPadRight();
	afx_msg void OnButtonPadLeft();
	afx_msg void OnButtonPadDown();
	afx_msg void OnButtonCUp();
	afx_msg void OnButtonCRight();
	afx_msg void OnButtonCLeft();
	afx_msg void OnButtonCDown();
	afx_msg void OnButtonB();
	afx_msg void OnButtonA();
	afx_msg void OnButtonTriggerLeft();
	afx_msg void OnButtonTriggerRight();
	afx_msg void OnButtonTriggerZ();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CConfigurationDlg::CConfigurationDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CConfigurationDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CConfigurationDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CConfigurationDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigurationDlg)
	DDX_Control(pDX, IDC_CHECK_KAILLERA_VIEWLOG, m_checkKailleraViewLog);
	DDX_Control(pDX, IDC_STATIC_KAILLERA_DLL, m_staticKailleraDll);
	DDX_Control(pDX, IDC_STATIC_KAILLERA_VERSION, m_staticKailleraVersion);
	DDX_Control(pDX, IDC_STATIC_KAILLERA_PLAYER, m_staticKailleraPlayer);
	DDX_Control(pDX, IDC_COMBO_KAILLERA_PLAYER, m_comboKailleraPlayer);
	DDX_Control(pDX, IDC_CHECK_KAILLERA, m_checkKaillera);
	DDX_Control(pDX, IDC_BUTTON_KEY_SETTING, m_buttonSetting);
	DDX_Control(pDX, IDC_COMBO_TYPE, m_comboDevice);
	DDX_Control(pDX, IDC_COMBO_CONTROLLER, m_comboControllers);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigurationDlg, CDialog)
	//{{AFX_MSG_MAP(CConfigurationDlg)
	ON_BN_CLICKED(IDC_BUTTON_ABOUT, OnButtonAbout)
	ON_BN_CLICKED(IDC_BUTTON_TEST, OnButtonTest)
	ON_CBN_SELENDOK(IDC_COMBO_CONTROLLER, OnSelendokComboController)
	ON_CBN_SELENDOK(IDC_COMBO_TYPE, OnSelendokComboDevice)
	ON_BN_CLICKED(IDC_BUTTON_KEY_SETTING, OnButtonKeySetting)
	ON_BN_CLICKED(IDC_CHECK_KAILLERA, OnCheckKaillera)
	ON_CBN_SELENDOK(IDC_COMBO_KAILLERA_PLAYER, OnSelendokComboKailleraPlayer)
	ON_BN_CLICKED(IDC_CHECK_KAILLERA_VIEWLOG, OnCheckKailleraViewLog)
	ON_BN_CLICKED(IDC_BUTTON_KAILLERA_DLL, OnButtonKailleraDll)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConfigurationDlg message handlers

void PrintConfigurationDialog(HWND hParent)
{
  AFX_MANAGE_STATE(AfxGetStaticModuleState());

  CWnd wParent;
  wParent.Attach(hParent);

  CConfigurationDlg dlg(&wParent);
  
  dlg.DoModal();

  wParent.Detach();
}

BOOL CConfigurationDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

  int idx = 0;
  idx = m_comboControllers.AddString(LABEL_N64Controller1);
  m_comboControllers.SetItemData(idx, 0);
  idx = m_comboControllers.AddString(LABEL_N64Controller2);
  m_comboControllers.SetItemData(idx, 1);
  idx = m_comboControllers.AddString(LABEL_N64Controller3);
  m_comboControllers.SetItemData(idx, 2);
  idx = m_comboControllers.AddString(LABEL_N64Controller4);
  m_comboControllers.SetItemData(idx, 3);

  idx = m_comboDevice.AddString(LABEL_Disable);
  m_comboDevice.SetItemData(idx , 0xFFFF);
	for(int i=0; i<MAX_DEVICES; i++)
	{
		if((DInputDevices[i].iType == DIDEVICE_KEYBOARD
			|| DInputDevices[i].iType == DIDEVICE_JOYSTICK)
			&& DInputDevices[i].pDevice != NULL)
		{
			idx = m_comboDevice.AddString(DInputDevices[i].szName);
			m_comboDevice.SetItemData(idx , i);
		}
	}
	
	if(!bKailleraDllLoaded)
		m_checkKaillera.EnableWindow(FALSE);
	else
		m_checkKaillera.EnableWindow(TRUE);
	m_staticKailleraDll.SetWindowText(g_szKailleraDll);
  m_checkKaillera.SetCheck(g_bUseKaillera?1:0);
	OnCheckKaillera();
  idx = m_comboKailleraPlayer.AddString(LABEL_N64Controller1);
	idx = m_comboKailleraPlayer.AddString(LABEL_N64Controller2);
	idx = m_comboKailleraPlayer.AddString(LABEL_N64Controller3);
	idx = m_comboKailleraPlayer.AddString(LABEL_N64Controller4);
	//idx = m_comboKailleraPlayer.AddString(LABEL_Spectator);
	m_comboKailleraPlayer.SetCurSel(g_iKailleraPlayAs);
	if(g_bKailleraViewLog)
		m_checkKailleraViewLog.SetCheck(1);
	else
		m_checkKailleraViewLog.SetCheck(0);
  
  m_comboControllers.SelectString(-1, LABEL_N64Controller1);
  OnSelendokComboController();

	char szKailleraVersion[16]="No version !";
	kailleraGetVersion(szKailleraVersion);
	m_staticKailleraVersion.SetWindowText(szKailleraVersion);

  return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CConfigurationDlg::OnButtonAbout() 
{
	PrintAboutBox(this);
}

void CConfigurationDlg::OnButtonTest() 
{
	PrintTestDialog(this);	
}

void CConfigurationDlg::OnOK() 
{
  WriteConfiguration();
}

void CConfigurationDlg::OnSelendokComboController() 
{
	int idx = m_comboControllers.GetCurSel();
	int nController = m_comboControllers.GetItemData(idx);

	if(Controllers[nController].diHandle != -1)
	{
    if(m_comboDevice.SelectString(-1, DInputDevices[Controllers[nController].diHandle].szName) == -1)
		{
			m_comboDevice.SelectString(-1, LABEL_Disable);
			m_buttonSetting.EnableWindow(FALSE);
		}
		else
			m_buttonSetting.EnableWindow(TRUE);
	}
	else
  {
    m_comboDevice.SelectString(-1, LABEL_Disable);
    m_buttonSetting.EnableWindow(FALSE);
  }
}

void CConfigurationDlg::OnSelendokComboDevice() 
{
	int idx = m_comboControllers.GetCurSel();
	int nController = m_comboControllers.GetItemData(idx);
  idx = m_comboDevice.GetCurSel();
  int nDevice = m_comboDevice.GetItemData(idx);

  if(nDevice < MAX_DEVICES)
  {
    Controllers[nController].diHandle = nDevice;
    m_buttonSetting.EnableWindow(TRUE);
  }
  else
  {
    Controllers[nController].diHandle = -1;
    m_buttonSetting.EnableWindow(FALSE);
  }
}

void CConfigurationDlg::OnButtonKeySetting() 
{
	int idx = m_comboControllers.GetCurSel();
	int nController = m_comboControllers.GetItemData(idx);
	
	if(Controllers[nController].diHandle != -1)
	{
		if(DInputDevices[Controllers[nController].diHandle].iType == DIDEVICE_KEYBOARD)
		{
			CKeyboardSetting dlg(this);
			
			dlg.controller = &Controllers[nController];
			
			dlg.DoModal();
		}
		else if(DInputDevices[Controllers[nController].diHandle].iType == DIDEVICE_JOYSTICK)
		{
			CJoystickSetting dlg(this);
			
			dlg.controller = &Controllers[nController];
			
			dlg.DoModal();
		}
	}
}

void CConfigurationDlg::OnCheckKaillera() 
{
	BOOL bEnable = m_checkKaillera.GetCheck()?TRUE:FALSE;

	m_staticKailleraPlayer.EnableWindow(bEnable);
	m_comboKailleraPlayer.EnableWindow(bEnable);
	m_checkKailleraViewLog.EnableWindow(bEnable);
	g_bUseKaillera = bEnable?true:false;
}

void CConfigurationDlg::OnSelendokComboKailleraPlayer() 
{
	g_iKailleraPlayAs = m_comboKailleraPlayer.GetCurSel();
}

void CConfigurationDlg::OnCheckKailleraViewLog() 
{
	g_bKailleraViewLog = m_checkKailleraViewLog.GetCheck()?true:false;
}

void CConfigurationDlg::OnButtonKailleraDll() 
{
	CString strIn(g_szKailleraDll);

	CFileDialog dlg(TRUE, NULL, strIn, OFN_FILEMUSTEXIST);

	if(dlg.DoModal() == IDOK)
	{
		strncpy(g_szKailleraDll, dlg.GetPathName(), sizeof(g_szKailleraDll));
		m_staticKailleraDll.SetWindowText(g_szKailleraDll);
	}
}

CJoystickSetting::CJoystickSetting(CWnd* pParent /*=NULL*/)
	: CDialog(CJoystickSetting::IDD, pParent)
{
	//{{AFX_DATA_INIT(CJoystickSetting)
	//}}AFX_DATA_INIT
}


void CJoystickSetting::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CJoystickSetting)
	DDX_Control(pDX, IDC_COMBO_STICK_UPDOWN, m_comboStickUpDown);
	DDX_Control(pDX, IDC_COMBO_STICK_LEFTRIGHT, m_comboStickLeftRight);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CJoystickSetting, CDialog)
	//{{AFX_MSG_MAP(CJoystickSetting)
	ON_BN_CLICKED(IDC_BUTTON_DEFAULT, OnButtonDefault)
	ON_BN_CLICKED(IDC_BUTTON_START, OnButtonStart)
	ON_BN_CLICKED(IDC_BUTTON_PAD_UP, OnButtonPadUp)
	ON_BN_CLICKED(IDC_BUTTON_PAD_RIGHT, OnButtonPadRight)
	ON_BN_CLICKED(IDC_BUTTON_PAD_LEFT, OnButtonPadLeft)
	ON_BN_CLICKED(IDC_BUTTON_PAD_DOWN, OnButtonPadDown)
	ON_BN_CLICKED(IDC_BUTTON_C_UP, OnButtonCUp)
	ON_BN_CLICKED(IDC_BUTTON_C_RIGHT, OnButtonCRight)
	ON_BN_CLICKED(IDC_BUTTON_C_LEFT, OnButtonCLeft)
	ON_BN_CLICKED(IDC_BUTTON_C_DOWN, OnButtonCDown)
	ON_BN_CLICKED(IDC_BUTTON_B, OnButtonB)
	ON_BN_CLICKED(IDC_BUTTON_A, OnButtonA)
	ON_BN_CLICKED(IDC_BUTTON_TRIGGER_LEFT, OnButtonTriggerLeft)
	ON_BN_CLICKED(IDC_BUTTON_TRIGGER_RIGHT, OnButtonTriggerRight)
	ON_BN_CLICKED(IDC_BUTTON_TRIGGER_Z, OnButtonTriggerZ)
	ON_CBN_SELENDOK(IDC_COMBO_STICK_UPDOWN, OnSelendokComboStickUpdown)
	ON_CBN_SELENDOK(IDC_COMBO_STICK_LEFTRIGHT, OnSelendokComboStickLeftright)
	ON_BN_CLICKED(IDC_BUTTON_STICK_DOWN, OnButtonStickDown)
	ON_BN_CLICKED(IDC_BUTTON_STICK_LEFT, OnButtonStickLeft)
	ON_BN_CLICKED(IDC_BUTTON_STICK_RIGHT, OnButtonStickRight)
	ON_BN_CLICKED(IDC_BUTTON_STICK_UP, OnButtonStickUp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CJoystickSetting message handlers

BOOL CJoystickSetting::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
  int idx;

  // axe UpDown
  idx = m_comboStickUpDown.AddString(LABEL_JoystickAxeX);
  m_comboStickUpDown.SetItemData(idx, 0);
  idx = m_comboStickUpDown.AddString(LABEL_JoystickInversedAxeX);
  m_comboStickUpDown.SetItemData(idx, 1);
  idx = m_comboStickUpDown.AddString(LABEL_JoystickAxeY);
  m_comboStickUpDown.SetItemData(idx, 2);
  idx = m_comboStickUpDown.AddString(LABEL_JoystickInversedAxeY);
  m_comboStickUpDown.SetItemData(idx, 3);
  idx = m_comboStickUpDown.AddString(LABEL_JoystickAxeZ);
  m_comboStickUpDown.SetItemData(idx, 4);
  idx = m_comboStickUpDown.AddString(LABEL_JoystickInversedAxeZ);
  m_comboStickUpDown.SetItemData(idx, 5);
  idx = m_comboStickUpDown.AddString(LABEL_UseButtons);
  m_comboStickUpDown.SetItemData(idx, 6);

  // axe LeftRight
  idx = m_comboStickLeftRight.AddString(LABEL_JoystickAxeX);
  m_comboStickLeftRight.SetItemData(idx, 0);
  idx = m_comboStickLeftRight.AddString(LABEL_JoystickInversedAxeX);
  m_comboStickLeftRight.SetItemData(idx, 1);
  idx = m_comboStickLeftRight.AddString(LABEL_JoystickAxeY);
  m_comboStickLeftRight.SetItemData(idx, 2);
  idx = m_comboStickLeftRight.AddString(LABEL_JoystickInversedAxeY);
  m_comboStickLeftRight.SetItemData(idx, 3);
  idx = m_comboStickLeftRight.AddString(LABEL_JoystickAxeZ);
  m_comboStickLeftRight.SetItemData(idx, 4);
  idx = m_comboStickLeftRight.AddString(LABEL_JoystickInversedAxeZ);
  m_comboStickLeftRight.SetItemData(idx, 5);
  idx = m_comboStickLeftRight.AddString(LABEL_UseButtons);
  m_comboStickLeftRight.SetItemData(idx, 6);

  SetMapping();

  OnSelendokComboStickUpdown();
  OnSelendokComboStickLeftright();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CJoystickSetting::OnButtonDefault() 
{
  SetAsDefaultJoystickController(controller, controller->diHandle);
  SetMapping();
}

void CJoystickSetting::SetMapping()
{
  for(int b=0; b<18; b++)
  {
    int key;
    switch(b)
    {
    case  0: key = controller->ButtonA; break;
    case  1: key = controller->ButtonB; break;
    case  2: key = controller->ButtonCDown; break;
    case  3: key = controller->ButtonCLeft; break;
    case  4: key = controller->ButtonCRight; break;
    case  5: key = controller->ButtonCUp; break;
    case  6: key = controller->ButtonStart; break;
    case  7: key = controller->PadDown; break;
    case  8: key = controller->PadLeft; break;
    case  9: key = controller->PadRight; break;
    case 10: key = controller->PadUp; break;
    case 11: key = controller->TriggerLeft; break;
    case 12: key = controller->TriggerRight; break;
    case 13: key = controller->TriggerZ; break;
    case 14: key = controller->StickDown; break;
    case 15: key = controller->StickLeft; break;
    case 16: key = controller->StickRight; break;
    case 17: key = controller->StickUp; break;
    }

    char label[64];
    sprintf(label, "Button %2d", key);

    switch(b)
    {
    case  0: ::SetWindowText(GetDlgItem(IDC_LABEL_A)->GetSafeHwnd(), label); break;
    case  1: ::SetWindowText(GetDlgItem(IDC_LABEL_B)->GetSafeHwnd(), label); break;
    case  2: ::SetWindowText(GetDlgItem(IDC_LABEL_C_DOWN)->GetSafeHwnd(), label); break;
    case  3: ::SetWindowText(GetDlgItem(IDC_LABEL_C_LEFT)->GetSafeHwnd(), label); break;
    case  4: ::SetWindowText(GetDlgItem(IDC_LABEL_C_RIGHT)->GetSafeHwnd(), label); break;
    case  5: ::SetWindowText(GetDlgItem(IDC_LABEL_C_UP)->GetSafeHwnd(), label); break;
    case  6: ::SetWindowText(GetDlgItem(IDC_LABEL_START)->GetSafeHwnd(), label); break;
    case  7: ::SetWindowText(GetDlgItem(IDC_LABEL_PAD_DOWN)->GetSafeHwnd(), label); break;
    case  8: ::SetWindowText(GetDlgItem(IDC_LABEL_PAD_LEFT)->GetSafeHwnd(), label); break;
    case  9: ::SetWindowText(GetDlgItem(IDC_LABEL_PAD_RIGHT)->GetSafeHwnd(), label); break;
    case 10: ::SetWindowText(GetDlgItem(IDC_LABEL_PAD_UP)->GetSafeHwnd(), label); break;
    case 11: ::SetWindowText(GetDlgItem(IDC_LABEL_TRIGGER_LEFT)->GetSafeHwnd(), label); break;
    case 12: ::SetWindowText(GetDlgItem(IDC_LABEL_TRIGGER_RIGHT)->GetSafeHwnd(), label); break;
    case 13: ::SetWindowText(GetDlgItem(IDC_LABEL_TRIGGER_Z)->GetSafeHwnd(), label); break;
    case 14: ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_DOWN)->GetSafeHwnd(), label); break;
    case 15: ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_LEFT)->GetSafeHwnd(), label); break;
    case 16: ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_RIGHT)->GetSafeHwnd(), label); break;
    case 17: ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_UP)->GetSafeHwnd(), label); break;
    }
  }
  if(controller->AxeForStickLeftRight != -1)
  {
    m_comboStickLeftRight.SetCurSel(controller->AxeForStickLeftRight);
    ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_LEFT)->GetSafeHwnd(), LABEL_Unused);
    ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_RIGHT)->GetSafeHwnd(), LABEL_Unused);
  }
  else
  {
    m_comboStickLeftRight.SelectString(-1, LABEL_UseButtons);
  }
  if(controller->AxeForStickUpDown != -1)
  {
    m_comboStickUpDown.SetCurSel(controller->AxeForStickUpDown);
    ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_UP)->GetSafeHwnd(), LABEL_Unused);
    ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_DOWN)->GetSafeHwnd(), LABEL_Unused);
  }
  else
  {
    m_comboStickUpDown.SelectString(-1, LABEL_UseButtons);
  }
}

void CJoystickSetting::OnButtonStart() 
{
	controller->ButtonStart = ButtonMapping(IDC_BUTTON_START);
  SetMapping();
}

void CJoystickSetting::OnButtonStickUp() 
{
	controller->StickUp = ButtonMapping(IDC_BUTTON_STICK_UP);
  SetMapping();
}

void CJoystickSetting::OnButtonStickRight() 
{
	controller->StickRight = ButtonMapping(IDC_BUTTON_STICK_RIGHT);
  SetMapping();
}

void CJoystickSetting::OnButtonStickLeft() 
{
	controller->StickLeft = ButtonMapping(IDC_BUTTON_STICK_LEFT);
  SetMapping();
}

void CJoystickSetting::OnButtonStickDown() 
{
	controller->StickDown = ButtonMapping(IDC_BUTTON_STICK_DOWN);
  SetMapping();
}

void CJoystickSetting::OnButtonPadUp() 
{
	controller->PadUp = ButtonMapping(IDC_BUTTON_PAD_UP);
  SetMapping();
}

void CJoystickSetting::OnButtonPadRight() 
{
	controller->PadRight = ButtonMapping(IDC_BUTTON_PAD_RIGHT);
  SetMapping();
}

void CJoystickSetting::OnButtonPadLeft() 
{
	controller->PadLeft = ButtonMapping(IDC_BUTTON_PAD_LEFT);
  SetMapping();
}

void CJoystickSetting::OnButtonPadDown() 
{
	controller->PadDown = ButtonMapping(IDC_BUTTON_PAD_DOWN);
  SetMapping();
}

void CJoystickSetting::OnSelendokComboStickUpdown() 
{
  int idx = m_comboStickUpDown.GetCurSel();
  controller->AxeForStickUpDown = m_comboStickUpDown.GetItemData(idx);
  if(controller->AxeForStickUpDown == 6)
  {
    ::EnableWindow(GetDlgItem(IDC_BUTTON_STICK_UP)->GetSafeHwnd(), TRUE);
    ::EnableWindow(GetDlgItem(IDC_BUTTON_STICK_DOWN)->GetSafeHwnd(), TRUE);
  }
  else
  {
    ::EnableWindow(GetDlgItem(IDC_BUTTON_STICK_UP)->GetSafeHwnd(), FALSE);
    ::EnableWindow(GetDlgItem(IDC_BUTTON_STICK_DOWN)->GetSafeHwnd(), FALSE);
  }
  SetMapping();
}

void CJoystickSetting::OnSelendokComboStickLeftright() 
{
  int idx = m_comboStickLeftRight.GetCurSel();
  controller->AxeForStickLeftRight = m_comboStickLeftRight.GetItemData(idx);
  if(controller->AxeForStickLeftRight == 6)
  {
    ::EnableWindow(GetDlgItem(IDC_BUTTON_STICK_RIGHT)->GetSafeHwnd(), TRUE);
    ::EnableWindow(GetDlgItem(IDC_BUTTON_STICK_LEFT)->GetSafeHwnd(), TRUE);
  }
  else
  {
    ::EnableWindow(GetDlgItem(IDC_BUTTON_STICK_RIGHT)->GetSafeHwnd(), FALSE);
    ::EnableWindow(GetDlgItem(IDC_BUTTON_STICK_LEFT)->GetSafeHwnd(), FALSE);
  }
  SetMapping();
}

void CJoystickSetting::OnButtonA() 
{
	controller->ButtonA = ButtonMapping(IDC_BUTTON_A);
  SetMapping();
}

void CJoystickSetting::OnButtonB() 
{
	controller->ButtonB = ButtonMapping(IDC_BUTTON_B);
  SetMapping();
}

void CJoystickSetting::OnButtonCUp() 
{
	controller->ButtonCUp = ButtonMapping(IDC_BUTTON_C_UP);
  SetMapping();
}

void CJoystickSetting::OnButtonCRight() 
{
	controller->ButtonCRight = ButtonMapping(IDC_BUTTON_C_RIGHT);
  SetMapping();
}

void CJoystickSetting::OnButtonCLeft() 
{
	controller->ButtonCLeft = ButtonMapping(IDC_BUTTON_C_LEFT);
  SetMapping();
}

void CJoystickSetting::OnButtonCDown() 
{
	controller->ButtonCDown = ButtonMapping(IDC_BUTTON_C_DOWN);
  SetMapping();
}

void CJoystickSetting::OnButtonTriggerLeft() 
{
	controller->TriggerLeft = ButtonMapping(IDC_BUTTON_TRIGGER_LEFT);
  SetMapping();
}

void CJoystickSetting::OnButtonTriggerRight() 
{
	controller->TriggerRight = ButtonMapping(IDC_BUTTON_TRIGGER_RIGHT);
  SetMapping();
}

void CJoystickSetting::OnButtonTriggerZ() 
{
	controller->TriggerZ = ButtonMapping(IDC_BUTTON_TRIGGER_Z);
  SetMapping();
}

__int32 CJoystickSetting::ButtonMapping(LONG idButton)
{
  IDirectInputDevice7*	pDevice = NULL;
  IDirectInput7*  			pInput = NULL;
  HRESULT ddrval;
  __int32 button = 0;

  ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), FALSE);
  
  ddrval = DirectInputCreateEx(AfxGetInstanceHandle(), DIRECTINPUT_VERSION, IID_IDirectInput7, (void**)&pInput, NULL);
  if (FAILED(ddrval))
  {
    Complain(GetSafeHwnd(), ddrval, "DirectInputCreate - couldn't initialize...");
    return FALSE;
  }
  
  ddrval = pInput->CreateDeviceEx(DInputDevices[controller->diHandle].Guid, IID_IDirectInputDevice7, (void**)&pDevice, NULL);
  if (FAILED(ddrval))
  {
    Complain(GetSafeHwnd(), ddrval, "CreateDevice : Joystick");
    ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), TRUE);
    pInput->Release();
    return FALSE;
  }
  ddrval = pDevice->SetDataFormat(&c_dfDIJoystick);
  if (FAILED(ddrval))
  {
    Complain(GetSafeHwnd(), ddrval, "SetDataFormat : Joystick");
    ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), TRUE);
    pDevice->Unacquire();
    pDevice->Release();
    pInput->Release();
    return FALSE;
  }
  ddrval = pDevice->SetCooperativeLevel(GetSafeHwnd(), DISCL_FOREGROUND | DISCL_NONEXCLUSIVE); 
  if (FAILED(ddrval))
  {
    Complain(GetSafeHwnd(), ddrval, "SetCooperativeLevel : Joystick");
    ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), TRUE);
    pDevice->Unacquire();
    pDevice->Release();
    pInput->Release();
    return FALSE;
  }
  if (pDevice) pDevice->Acquire();

  DIJOYSTATE js;

  bool find = false;
  while(!find)
  {
    memset (&js, 0, sizeof(js));
    pDevice->Poll();
    
againAcquire:
    
    ddrval = pDevice->GetDeviceState(sizeof(DIJOYSTATE), &js);
    if (ddrval == DIERR_INPUTLOST)
    {
      ddrval = pDevice->Acquire();
      goto againAcquire;
    }
    for(int i=0; i<32; i++)
    {
      if(js.rgbButtons[i] & 0x80)
      {
        button = i;
        find = true;
        break;
      }
    }
  }

  pDevice->Unacquire();
  pDevice->Release();
  pInput->Release();
  
  ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), TRUE);

  return button;
}

/////////////////////////////////////////////////////////////////////////////
// CKeyboardSetting dialog


CKeyboardSetting::CKeyboardSetting(CWnd* pParent /*=NULL*/)
	: CDialog(CKeyboardSetting::IDD, pParent)
{
	//{{AFX_DATA_INIT(CKeyboardSetting)
	//}}AFX_DATA_INIT
}


void CKeyboardSetting::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CKeyboardSetting)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CKeyboardSetting, CDialog)
	//{{AFX_MSG_MAP(CKeyboardSetting)
	ON_BN_CLICKED(IDC_BUTTON_DEFAULT, OnButtonDefault)
	ON_BN_CLICKED(IDC_BUTTON_START, OnButtonStart)
	ON_BN_CLICKED(IDC_BUTTON_STICK_UP, OnButtonStickUp)
	ON_BN_CLICKED(IDC_BUTTON_STICK_RIGHT, OnButtonStickRight)
	ON_BN_CLICKED(IDC_BUTTON_STICK_LEFT, OnButtonStickLeft)
	ON_BN_CLICKED(IDC_BUTTON_STICK_DOWN, OnButtonStickDown)
	ON_BN_CLICKED(IDC_BUTTON_PAD_UP, OnButtonPadUp)
	ON_BN_CLICKED(IDC_BUTTON_PAD_RIGHT, OnButtonPadRight)
	ON_BN_CLICKED(IDC_BUTTON_PAD_LEFT, OnButtonPadLeft)
	ON_BN_CLICKED(IDC_BUTTON_PAD_DOWN, OnButtonPadDown)
	ON_BN_CLICKED(IDC_BUTTON_C_UP, OnButtonCUp)
	ON_BN_CLICKED(IDC_BUTTON_C_RIGHT, OnButtonCRight)
	ON_BN_CLICKED(IDC_BUTTON_C_LEFT, OnButtonCLeft)
	ON_BN_CLICKED(IDC_BUTTON_C_DOWN, OnButtonCDown)
	ON_BN_CLICKED(IDC_BUTTON_B, OnButtonB)
	ON_BN_CLICKED(IDC_BUTTON_A, OnButtonA)
	ON_BN_CLICKED(IDC_BUTTON_TRIGGER_LEFT, OnButtonTriggerLeft)
	ON_BN_CLICKED(IDC_BUTTON_TRIGGER_RIGHT, OnButtonTriggerRight)
	ON_BN_CLICKED(IDC_BUTTON_TRIGGER_Z, OnButtonTriggerZ)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKeyboardSetting message handlers

BOOL CKeyboardSetting::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	SetMapping();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CKeyboardSetting::OnButtonDefault() 
{
  SetAsDefaultKeyboardController(controller);
  SetMapping();	
}

void CKeyboardSetting::SetMapping()
{
  for(int b=0; b<18; b++)
  {
    int key;
    switch(b)
    {
    case  0: key = controller->ButtonA; break;
    case  1: key = controller->ButtonB; break;
    case  2: key = controller->ButtonCDown; break;
    case  3: key = controller->ButtonCLeft; break;
    case  4: key = controller->ButtonCRight; break;
    case  5: key = controller->ButtonCUp; break;
    case  6: key = controller->ButtonStart; break;
    case  7: key = controller->PadDown; break;
    case  8: key = controller->PadLeft; break;
    case  9: key = controller->PadRight; break;
    case 10: key = controller->PadUp; break;
    case 11: key = controller->StickDown; break;
    case 12: key = controller->StickLeft; break;
    case 13: key = controller->StickRight; break;
    case 14: key = controller->StickUp; break;
    case 15: key = controller->TriggerLeft; break;
    case 16: key = controller->TriggerRight; break;
    case 17: key = controller->TriggerZ; break;
    }

    char label[64];
    switch(key)
    {
    case DIK_ESCAPE: sprintf(label, "DIK_ESCAPE"); break;
    case DIK_1: sprintf(label, "DIK_1"); break;
    case DIK_2: sprintf(label, "DIK_2"); break;
    case DIK_3: sprintf(label, "DIK_3"); break;
    case DIK_4: sprintf(label, "DIK_4"); break;
    case DIK_5: sprintf(label, "DIK_5"); break;
    case DIK_6: sprintf(label, "DIK_6"); break;
    case DIK_7: sprintf(label, "DIK_7"); break;
    case DIK_8: sprintf(label, "DIK_8"); break;
    case DIK_9: sprintf(label, "DIK_9"); break;
    case DIK_0: sprintf(label, "DIK_0"); break;
    case DIK_MINUS: sprintf(label, "DIK_MINUS"); break;
    case DIK_EQUALS: sprintf(label, "DIK_EQUAL"); break;
    case DIK_BACK: sprintf(label, "DIK_BACK"); break;
    case DIK_TAB: sprintf(label, "DIK_TAB"); break;
    case DIK_Q: sprintf(label, "DIK_Q"); break;
    case DIK_W: sprintf(label, "DIK_W"); break;
    case DIK_E: sprintf(label, "DIK_E"); break;
    case DIK_R: sprintf(label, "DIK_R"); break;
    case DIK_T: sprintf(label, "DIK_T"); break;
    case DIK_Y: sprintf(label, "DIK_Y"); break;
    case DIK_U: sprintf(label, "DIK_U"); break;
    case DIK_I: sprintf(label, "DIK_I"); break;
    case DIK_O: sprintf(label, "DIK_O"); break;
    case DIK_P: sprintf(label, "DIK_P"); break;
    case DIK_LBRACKET: sprintf(label, "DIK_LBRACKET"); break;
    case DIK_RBRACKET: sprintf(label, "DIK_RBRACKET"); break;
    case DIK_RETURN: sprintf(label, "DIK_RETURN"); break;
    case DIK_LCONTROL: sprintf(label, "DIK_LCONTROL"); break;
    case DIK_A: sprintf(label, "DIK_A"); break;
    case DIK_S: sprintf(label, "DIK_S"); break;
    case DIK_D: sprintf(label, "DIK_D"); break;
    case DIK_F: sprintf(label, "DIK_F"); break;
    case DIK_G: sprintf(label, "DIK_G"); break;
    case DIK_H: sprintf(label, "DIK_H"); break;
    case DIK_J: sprintf(label, "DIK_J"); break;
    case DIK_K: sprintf(label, "DIK_K"); break;
    case DIK_L: sprintf(label, "DIK_L"); break;
    case DIK_SEMICOLON: sprintf(label, "DIK_SEMICOLON"); break;
    case DIK_APOSTROPHE: sprintf(label, "DIK_APOSTROPHE"); break;
    case DIK_GRAVE: sprintf(label, "DIK_GRAVE"); break;
    case DIK_LSHIFT: sprintf(label, "DIK_LSHIFT"); break;
    case DIK_BACKSLASH: sprintf(label, "DIK_BACKSLASH"); break;
    case DIK_Z: sprintf(label, "DIK_Z"); break;
    case DIK_X: sprintf(label, "DIK_X"); break;
    case DIK_C: sprintf(label, "DIK_C"); break;
    case DIK_V: sprintf(label, "DIK_V"); break;
    case DIK_B: sprintf(label, "DIK_B"); break;
    case DIK_N: sprintf(label, "DIK_N"); break;
    case DIK_M: sprintf(label, "DIK_M"); break;
    case DIK_COMMA: sprintf(label, "DIK_COMMA"); break;
    case DIK_PERIOD: sprintf(label, "DIK_PERIOD"); break;
    case DIK_SLASH: sprintf(label, "DIK_SLASH"); break;
    case DIK_RSHIFT: sprintf(label, "DIK_RSHIFT"); break;
    case DIK_MULTIPLY: sprintf(label, "DIK_MULTIPlY"); break;
    case DIK_LMENU: sprintf(label, "DIK_LMENU"); break;
    case DIK_SPACE: sprintf(label, "DIK_SPACE"); break;
    case DIK_CAPITAL: sprintf(label, "DIK_CAPITAL"); break;
    case DIK_F1: sprintf(label, "DIK_F1"); break;
    case DIK_F2: sprintf(label, "DIK_F2"); break;
    case DIK_F3: sprintf(label, "DIK_F3"); break;
    case DIK_F4: sprintf(label, "DIK_F4"); break;
    case DIK_F5: sprintf(label, "DIK_F5"); break;
    case DIK_F6: sprintf(label, "DIK_F6"); break;
    case DIK_F7: sprintf(label, "DIK_F7"); break;
    case DIK_F8: sprintf(label, "DIK_F8"); break;
    case DIK_F9: sprintf(label, "DIK_F9"); break;
    case DIK_F10: sprintf(label, "DIKF_10"); break;
    case DIK_NUMLOCK: sprintf(label, "DIK_NUMLOCK"); break;
    case DIK_SCROLL: sprintf(label, "DIK_SCROLL"); break;
    case DIK_NUMPAD7: sprintf(label, "DIK_NUMPAD7"); break;
    case DIK_NUMPAD8: sprintf(label, "DIK_NUMPAD8"); break;
    case DIK_NUMPAD9: sprintf(label, "DIK_NUMPAD9"); break;
    case DIK_SUBTRACT: sprintf(label, "DIK_SUBSTRACT"); break;
    case DIK_NUMPAD4: sprintf(label, "DIK_NUMPAD4"); break;
    case DIK_NUMPAD5: sprintf(label, "DIK_NUMPAD5"); break;
    case DIK_NUMPAD6: sprintf(label, "DIK_NUMPAD6"); break;
    case DIK_ADD: sprintf(label, "DIK_ADD"); break;
    case DIK_NUMPAD1: sprintf(label, "DIK_NUMPAD1"); break;
    case DIK_NUMPAD2: sprintf(label, "DIK_NUMPAD2"); break;
    case DIK_NUMPAD3: sprintf(label, "DIK_NUMPAD3"); break;
    case DIK_NUMPAD0: sprintf(label, "DIK_NUMPAD0"); break;
    case DIK_DECIMAL: sprintf(label, "DIK_DECIMAL"); break;
    case DIK_OEM_102: sprintf(label, "DIK_OEM_102"); break;
    case DIK_F11: sprintf(label, "DIK_F11"); break;
    case DIK_F12: sprintf(label, "DIK_F12"); break;
    case DIK_F13: sprintf(label, "DIK_F13"); break;
    case DIK_F14: sprintf(label, "DIK_F14"); break;
    case DIK_F15: sprintf(label, "DIK_F15"); break;
    case DIK_KANA: sprintf(label, "DIK_KANA"); break;
    case DIK_ABNT_C1: sprintf(label, "DIK_ADNT_C1"); break;
    case DIK_CONVERT: sprintf(label, "DIK_CONVERT"); break;
    case DIK_NOCONVERT: sprintf(label, "DIK_NOCONVERT"); break;
    case DIK_YEN: sprintf(label, "DIK_YEN"); break;
    case DIK_ABNT_C2: sprintf(label, "DIK_ABNT_C2"); break;
    case DIK_NUMPADEQUALS: sprintf(label, "DIK_NUMPADEQUALS"); break;
    case DIK_PREVTRACK: sprintf(label, "DIK_PREVTRACK"); break;
    case DIK_AT: sprintf(label, "DIK_AT"); break;
    case DIK_COLON: sprintf(label, "DIK_COLON"); break;
    case DIK_UNDERLINE: sprintf(label, "DIK_UNDERLINE"); break;
    case DIK_KANJI: sprintf(label, "DIK_KANJI"); break;
    case DIK_STOP: sprintf(label, "DIK_STOP"); break;
    case DIK_AX: sprintf(label, "DIK_AX"); break;
    case DIK_UNLABELED: sprintf(label, "DIK_UNLABELED"); break;
    case DIK_NEXTTRACK: sprintf(label, "DIK_NEXTTRACK"); break;
    case DIK_NUMPADENTER: sprintf(label, "DIK_NUMPADENTER"); break;
    case DIK_RCONTROL: sprintf(label, "DIK_RCONTROL"); break;
    case DIK_MUTE: sprintf(label, "DIK_MUTE"); break;
    case DIK_CALCULATOR: sprintf(label, "DIK_CALCULATOR"); break;
    case DIK_PLAYPAUSE: sprintf(label, "DIK_PLAYPAUSE"); break;
    case DIK_MEDIASTOP: sprintf(label, "DIK_MEDIASTOP"); break;
    case DIK_VOLUMEDOWN: sprintf(label, "DIK_VOLUMEDOWN"); break;
    case DIK_VOLUMEUP: sprintf(label, "DIK_VOLUMEUP"); break;
    case DIK_WEBHOME: sprintf(label, "DIK_WEBHOME"); break;
    case DIK_NUMPADCOMMA: sprintf(label, "DIK_NUMPADCOMMA"); break;
    case DIK_DIVIDE: sprintf(label, "DIK_DIVIDE"); break;
    case DIK_SYSRQ: sprintf(label, "DIK_SYSRQ"); break;
    case DIK_RMENU: sprintf(label, "DIK_RMENU"); break;
    case DIK_PAUSE: sprintf(label, "DIK_PAUSE"); break;
    case DIK_HOME: sprintf(label, "DIK_HOME"); break;
    case DIK_UP: sprintf(label, "DIK_UP"); break;
    case DIK_PRIOR: sprintf(label, "DIK_PRIOR"); break;
    case DIK_LEFT: sprintf(label, "DIK_LEFT"); break;
    case DIK_RIGHT: sprintf(label, "DIK_RIGHT"); break;
    case DIK_END: sprintf(label, "DIK_END"); break;
    case DIK_DOWN: sprintf(label, "DIK_DOWN"); break;
    case DIK_NEXT: sprintf(label, "DIK_NEXT"); break;
    case DIK_INSERT: sprintf(label, "DIK_INSERT"); break;
    case DIK_DELETE: sprintf(label, "DIK_DELETE"); break;
    case DIK_LWIN: sprintf(label, "DIK_LWIN"); break;
    case DIK_RWIN: sprintf(label, "DIK_RWIN"); break;
    case DIK_APPS: sprintf(label, "DIK_APPS"); break;
    case DIK_POWER: sprintf(label, "DIK_POWER"); break;
    case DIK_SLEEP: sprintf(label, "DIK_SLEEP"); break;
    case DIK_WAKE: sprintf(label, "DIK_WAKE"); break;
    case DIK_WEBSEARCH: sprintf(label, "DIK_WEBSEARCH"); break;
    case DIK_WEBFAVORITES: sprintf(label, "DIK_WEBFAVORITES"); break;
    case DIK_WEBREFRESH: sprintf(label, "DIK_WEBREFRESH"); break;
    case DIK_WEBSTOP: sprintf(label, "DIK_WEBSTOP"); break;
    case DIK_WEBFORWARD: sprintf(label, "DIK_WEBFORWARD"); break;
    case DIK_WEBBACK: sprintf(label, "DIK_WEBBACK"); break;
    case DIK_MYCOMPUTER: sprintf(label, "DIK_MYCOMPUTER"); break;
    case DIK_MAIL: sprintf(label, "DIK_MAIL"); break;
    case DIK_MEDIASELECT: sprintf(label, "DIK_MEDIASELECT"); break;
    default:
      sprintf(label, "DIK_(%3d)", key);
      break;
    }

    switch(b)
    {
    case  0: ::SetWindowText(GetDlgItem(IDC_LABEL_A)->GetSafeHwnd(), label); break;
    case  1: ::SetWindowText(GetDlgItem(IDC_LABEL_B)->GetSafeHwnd(), label); break;
    case  2: ::SetWindowText(GetDlgItem(IDC_LABEL_C_DOWN)->GetSafeHwnd(), label); break;
    case  3: ::SetWindowText(GetDlgItem(IDC_LABEL_C_LEFT)->GetSafeHwnd(), label); break;
    case  4: ::SetWindowText(GetDlgItem(IDC_LABEL_C_RIGHT)->GetSafeHwnd(), label); break;
    case  5: ::SetWindowText(GetDlgItem(IDC_LABEL_C_UP)->GetSafeHwnd(), label); break;
    case  6: ::SetWindowText(GetDlgItem(IDC_LABEL_START)->GetSafeHwnd(), label); break;
    case  7: ::SetWindowText(GetDlgItem(IDC_LABEL_PAD_DOWN)->GetSafeHwnd(), label); break;
    case  8: ::SetWindowText(GetDlgItem(IDC_LABEL_PAD_LEFT)->GetSafeHwnd(), label); break;
    case  9: ::SetWindowText(GetDlgItem(IDC_LABEL_PAD_RIGHT)->GetSafeHwnd(), label); break;
    case 10: ::SetWindowText(GetDlgItem(IDC_LABEL_PAD_UP)->GetSafeHwnd(), label); break;
    case 11: ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_DOWN)->GetSafeHwnd(), label); break;
    case 12: ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_LEFT)->GetSafeHwnd(), label); break;
    case 13: ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_RIGHT)->GetSafeHwnd(), label); break;
    case 14: ::SetWindowText(GetDlgItem(IDC_LABEL_STICK_UP)->GetSafeHwnd(), label); break;
    case 15: ::SetWindowText(GetDlgItem(IDC_LABEL_TRIGGER_LEFT)->GetSafeHwnd(), label); break;
    case 16: ::SetWindowText(GetDlgItem(IDC_LABEL_TRIGGER_RIGHT)->GetSafeHwnd(), label); break;
    case 17: ::SetWindowText(GetDlgItem(IDC_LABEL_TRIGGER_Z)->GetSafeHwnd(), label); break;
    }
  }
}

void CKeyboardSetting::OnButtonStart() 
{
	controller->ButtonStart = KeyMapping(IDC_BUTTON_START);
  SetMapping();
}

void CKeyboardSetting::OnButtonStickUp() 
{
	controller->StickUp = KeyMapping(IDC_BUTTON_STICK_UP);
  SetMapping();
}

void CKeyboardSetting::OnButtonStickRight() 
{
	controller->StickRight = KeyMapping(IDC_BUTTON_STICK_RIGHT);
  SetMapping();
}

void CKeyboardSetting::OnButtonStickLeft() 
{
	controller->StickLeft = KeyMapping(IDC_BUTTON_STICK_LEFT);
  SetMapping();
}

void CKeyboardSetting::OnButtonStickDown() 
{
	controller->StickDown = KeyMapping(IDC_BUTTON_STICK_DOWN);
  SetMapping();
}

void CKeyboardSetting::OnButtonPadUp() 
{
	controller->PadUp = KeyMapping(IDC_BUTTON_PAD_UP);
  SetMapping();
}

void CKeyboardSetting::OnButtonPadRight() 
{
	controller->PadRight = KeyMapping(IDC_BUTTON_PAD_RIGHT);
  SetMapping();
}

void CKeyboardSetting::OnButtonPadLeft() 
{
	controller->PadLeft = KeyMapping(IDC_BUTTON_PAD_LEFT);
  SetMapping();
}

void CKeyboardSetting::OnButtonPadDown() 
{
	controller->PadDown = KeyMapping(IDC_BUTTON_PAD_DOWN);
  SetMapping();
}

void CKeyboardSetting::OnButtonA() 
{
	controller->ButtonA = KeyMapping(IDC_BUTTON_A);
  SetMapping();
}

void CKeyboardSetting::OnButtonB() 
{
	controller->ButtonB = KeyMapping(IDC_BUTTON_B);
  SetMapping();
}

void CKeyboardSetting::OnButtonCUp() 
{
	controller->ButtonCUp = KeyMapping(IDC_BUTTON_C_UP);
  SetMapping();
}

void CKeyboardSetting::OnButtonCRight() 
{
	controller->ButtonCRight = KeyMapping(IDC_BUTTON_C_RIGHT);
  SetMapping();
}

void CKeyboardSetting::OnButtonCLeft() 
{
	controller->ButtonCLeft = KeyMapping(IDC_BUTTON_C_LEFT);
  SetMapping();
}

void CKeyboardSetting::OnButtonCDown() 
{
	controller->ButtonCDown = KeyMapping(IDC_BUTTON_C_DOWN);
  SetMapping();
}

void CKeyboardSetting::OnButtonTriggerLeft() 
{
	controller->TriggerLeft = KeyMapping(IDC_BUTTON_TRIGGER_LEFT);
  SetMapping();
}

void CKeyboardSetting::OnButtonTriggerRight() 
{
	controller->TriggerRight = KeyMapping(IDC_BUTTON_TRIGGER_RIGHT);
  SetMapping();
}

void CKeyboardSetting::OnButtonTriggerZ() 
{
	controller->TriggerZ = KeyMapping(IDC_BUTTON_TRIGGER_Z);
  SetMapping();
}

__int32 CKeyboardSetting::KeyMapping(LONG idButton)
{
  IDirectInputDevice7*	pDevice = NULL;
  IDirectInput7*  			pInput = NULL;
  HRESULT ddrval;
  
  ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), FALSE);
  
  ddrval = DirectInputCreateEx(AfxGetInstanceHandle(), DIRECTINPUT_VERSION, IID_IDirectInput7, (void**)&pInput, NULL);
  if (FAILED(ddrval))
  {
    Complain(GetSafeHwnd(), ddrval, "DirectInputCreate - couldn't initialize...");
    ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), TRUE);
    return FALSE;
  }
  
  ddrval = pInput->CreateDeviceEx(GUID_SysKeyboard, IID_IDirectInputDevice7, (void**)&pDevice, NULL); 
  if (FAILED(ddrval))
  {
    Complain(GetSafeHwnd(), ddrval, "CreateDevice : Keyboard");
    ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), TRUE);
    pInput->Release();
    return FALSE;
  }
  ddrval = pDevice->SetDataFormat(&c_dfDIKeyboard); 
  if (FAILED(ddrval))
  {
    Complain(GetSafeHwnd(), ddrval, "SetDataFormat : Keyboard");
    ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), TRUE);
    pDevice->Unacquire();
    pDevice->Release();
    pInput->Release();
    return FALSE;
  }
  ddrval = pDevice->SetCooperativeLevel(GetSafeHwnd(), DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);
  if (FAILED(ddrval))
  {
    Complain(GetSafeHwnd(), ddrval, "SetCooperativeLevel : Keyboard");
    ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), TRUE);
    pDevice->Unacquire();
    pDevice->Release();
    pInput->Release();
    return FALSE;
  }
  if (pDevice) pDevice->Acquire();

  __int32 key = -1;
  char     buffer[256]; 

  bool find = false;
  while(!find)
  {
    memset (&buffer,0,sizeof(buffer));
    
againAcquire:
    
    ddrval = pDevice->GetDeviceState(sizeof(buffer),(LPVOID)&buffer); 
    if (ddrval == DIERR_INPUTLOST)
    {
      ddrval = pDevice->Acquire();
      goto againAcquire;
    }
    for(int i=0; i<256; i++)
    {
      if(buffer[i] & 0x80)
      {
        key = i;
        find = true;
        break;
      }
    }
  }

  pDevice->Unacquire();
  pDevice->Release();
  pInput->Release();
  
  ::EnableWindow(GetDlgItem(idButton)->GetSafeHwnd(), TRUE);

  return key;
}
