#include "stdafx.h"
#include "noote_di.h"
#include <dinput.h>

#include "di.h"
#include "controller.h"
#include "config.h"
#include "kaillera.h"

IDirectInput7*  			pDInput = NULL;
DInputDevice DInputDevices[MAX_DEVICES];

HWND hWndInput = NULL;
bool bIsDIOpened = false;
unsigned long iGetKeysCounter=0;

static void Complain(HWND hwndOwner,HRESULT hr,LPCSTR pszMessage)
{
  MessageBox(hwndOwner, pszMessage, "NooTe_DI Error!", MB_OK);
}

void Void_DI()
{
	for(int i=0; i<MAX_DEVICES; i++)
	{
		DInputDevices[i].pDevice = NULL;
		DInputDevices[i].iType = 0;
  }
}

void Close_DI()
{
	for(int i=0; i<MAX_DEVICES; i++)
	{
		if (DInputDevices[i].pDevice != NULL)
		{
			DInputDevices[i].pDevice->Unacquire();
			DInputDevices[i].pDevice->Release();
			DInputDevices[i].pDevice = NULL;
		}
		DInputDevices[i].iType = 0;
  }
  
  if (pDInput)
  {
    pDInput->Release();
    pDInput = NULL;
  }

  bIsDIOpened = false;
}

BOOL CALLBACK DIEnumDevicesCallback(LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef)
{
  int *pnDevices = (int *)pvRef;
  int nDevices = *pnDevices;
  HRESULT ddrval;

	if(nDevices>=MAX_DEVICES)
		return DIENUM_STOP;

	BOOL bOk = true;
  if((lpddi->dwDevType & DIDEVTYPE_JOYSTICK) == DIDEVTYPE_JOYSTICK)
	{
		memcpy(&DInputDevices[nDevices].Guid, &lpddi->guidInstance, sizeof(GUID));
		strcpy(DInputDevices[nDevices].szName, lpddi->tszProductName);
		
		ddrval = pDInput->CreateDeviceEx(lpddi->guidInstance, IID_IDirectInputDevice7, (void**)&DInputDevices[nDevices].pDevice, NULL);
		if (!FAILED(ddrval) && DInputDevices[nDevices].pDevice != NULL)
		{
			ddrval = DInputDevices[nDevices].pDevice->SetDataFormat(&c_dfDIJoystick); 
			if (FAILED(ddrval))
			{
				Complain(hWndInput, ddrval, "SetDataFormat : Joystick");
				bOk = false;
			}
			//ddrval = DInputDevices[nDevices].pDevice->SetCooperativeLevel(hWndInput, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE); 
			ddrval = DInputDevices[nDevices].pDevice->SetCooperativeLevel(hWndInput, DISCL_BACKGROUND | DISCL_NONEXCLUSIVE);
			if (FAILED(ddrval))
			{
				Complain(hWndInput, ddrval, "SetCooperativeLevel : Joystick");
				bOk = false;
			}
			DIPROPRANGE diprg;
			diprg.diph.dwSize       = sizeof(diprg); 
			diprg.diph.dwHeaderSize = sizeof(diprg.diph); 
			diprg.diph.dwObj        = DIJOFS_X;
			diprg.diph.dwHow        = DIPH_BYOFFSET; 
			diprg.lMin              = -64;
			diprg.lMax              = +64;
			if (FAILED(DInputDevices[nDevices].pDevice->SetProperty(DIPROP_RANGE, &diprg.diph)))
			{ 
				Complain(hWndInput, ddrval, "SetProperty(DIPH_RANGE_X) : Joystick");
				bOk = false;
			}
			diprg.diph.dwObj        = DIJOFS_Y;
			if (FAILED(DInputDevices[nDevices].pDevice->SetProperty(DIPROP_RANGE, &diprg.diph)))
			{ 
				Complain(hWndInput, ddrval, "SetProperty(DIPH_RANGE_Y) : Joystick");
				bOk = false;
			}
			DInputDevices[nDevices].iType = DIDEVICE_JOYSTICK;
		}
		else
			Complain(hWndInput, ddrval, "CreateDevice : Joystick");
	}
  else if((lpddi->dwDevType & DIDEVTYPE_KEYBOARD) == DIDEVTYPE_KEYBOARD)
	{
		memcpy(&DInputDevices[nDevices].Guid, &lpddi->guidInstance, sizeof(GUID));
		strcpy(DInputDevices[nDevices].szName, lpddi->tszProductName);
		
		ddrval = pDInput->CreateDeviceEx(lpddi->guidInstance, IID_IDirectInputDevice7, (void**)&DInputDevices[nDevices].pDevice, NULL);
		if (!FAILED(ddrval) && DInputDevices[nDevices].pDevice != NULL)
		{
			ddrval = DInputDevices[nDevices].pDevice->SetDataFormat(&c_dfDIKeyboard); 
			if (FAILED(ddrval))
			{
				Complain(hWndInput, ddrval, "SetDataFormat : Keyboard");
				bOk = false;
			}
			ddrval = DInputDevices[nDevices].pDevice->SetCooperativeLevel(hWndInput, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE); 
			if (FAILED(ddrval))
			{
				Complain(hWndInput, ddrval, "SetCooperativeLevel : Keyboard");
				bOk = false;
			}
			DInputDevices[nDevices].iType = DIDEVICE_KEYBOARD;
		}
		else
			Complain(hWndInput, ddrval, "CreateDevice : Keyboard");
	}
	if(bOk)
	{
		DInputDevices[nDevices].pDevice->Acquire();
		(*pnDevices)++;
	}
	else
	{
		DInputDevices[nDevices].pDevice->Release();
		DInputDevices[nDevices].pDevice = NULL;
		DInputDevices[nDevices].iType = 0;
	}
  
  return DIENUM_CONTINUE;
}

void Open_DI(HWND hwnd)
{
  HRESULT ddrval;
  
	if(bIsDIOpened)
    return;
  
  Close_DI();

  ddrval = DirectInputCreateEx(AfxGetInstanceHandle(), DIRECTINPUT_VERSION, IID_IDirectInput7, (void**)&pDInput, NULL);
  if (FAILED(ddrval))
  {
    Complain(hWndInput, ddrval, "DirectInputCreate - couldn't initialize...");
    return;
  }
  
  int nDevices = 0;
	pDInput->EnumDevices(DIDEVTYPE_KEYBOARD, DIEnumDevicesCallback, (void*)&nDevices, DIEDFL_ATTACHEDONLY);
	pDInput->EnumDevices(DIDEVTYPE_JOYSTICK, DIEnumDevicesCallback, (void*)&nDevices, DIEDFL_ATTACHEDONLY);
	if(nDevices == 0)
		AfxMessageBox("No device found");
  bIsDIOpened = true;
}

void GetButtonsFrom(ControllerDef *pController, BUTTONS *pKeys)
{
#define KEYDOWN(buffer,key) (buffer[key] & 0x80) 

	if(pController->diHandle == -1 || pController->diHandle > MAX_DEVICES)
    return;

  DInputDevice *pDevice = &DInputDevices[pController->diHandle];

  WORD		   button = 0;
	signed char stick_x = 0;
	signed char stick_y = 0;

	if(pDevice->iType == DIDEVICE_KEYBOARD)
	{
		char buffer[256];
		HRESULT  ddrval;

		memset (&buffer,0,sizeof(buffer));

againKeyboardAcquire:
		ddrval = pDevice->pDevice->GetDeviceState(sizeof(buffer),(LPVOID)&buffer); 
		if (ddrval == DIERR_INPUTLOST || ddrval == DIERR_NOTACQUIRED)
		{
			ddrval = pDevice->pDevice->Acquire();
			if(ddrval != DIERR_OTHERAPPHASPRIO)
			{
				if(ddrval == DIERR_NOTINITIALIZED 
					&& ddrval == DIERR_INVALIDPARAM)
        {
          AfxMessageBox("Argh, Keyboard Error : NotInitialized or InvalideParam!!!");
					return;
        }
				goto againKeyboardAcquire;
			}
		}

		if(KEYDOWN(buffer, pController->StickUp))			  stick_y = 64;
		if(KEYDOWN(buffer, pController->StickDown))		  stick_y = -64;
		if(KEYDOWN(buffer, pController->StickLeft))		  stick_x = -64;
		if(KEYDOWN(buffer, pController->StickRight))		stick_x = 64;

		if(KEYDOWN(buffer, pController->PadUp))		      button |= U_JPAD;
		if(KEYDOWN(buffer, pController->PadDown))		    button |= D_JPAD;
		if(KEYDOWN(buffer, pController->PadLeft))		    button |= L_JPAD;
		if(KEYDOWN(buffer, pController->PadRight))		  button |= R_JPAD;

		if(KEYDOWN(buffer, pController->ButtonA))	      button |= A_BUTTON;
		if(KEYDOWN(buffer, pController->ButtonB))		    button |= B_BUTTON;

		if(KEYDOWN(buffer, pController->ButtonCUp))			button |= U_CBUTTONS;
		if(KEYDOWN(buffer, pController->ButtonCDown))		button |= D_CBUTTONS;
		if(KEYDOWN(buffer, pController->ButtonCLeft))		button |= L_CBUTTONS;
		if(KEYDOWN(buffer, pController->ButtonCRight))	button |= R_CBUTTONS;

		if(KEYDOWN(buffer, pController->ButtonStart))		button |= START_BUTTON;

		if(KEYDOWN(buffer, pController->TriggerLeft))		button |= L_TRIG;
		if(KEYDOWN(buffer, pController->TriggerRight))  button |= R_TRIG;
		if(KEYDOWN(buffer, pController->TriggerZ))			button |= Z_TRIG;
	}
  else if(pDevice->iType == DIDEVICE_JOYSTICK)
	{
		DIJOYSTATE js; 
		HRESULT  ddrval;

		pDevice->pDevice->Poll();

againJoystickAcquire:
		ddrval = pDevice->pDevice->GetDeviceState(sizeof(DIJOYSTATE), &js); 
		if (ddrval == DIERR_INPUTLOST || ddrval == DIERR_NOTACQUIRED)
		{
			ddrval = pDevice->pDevice->Acquire();
			if(ddrval != DIERR_OTHERAPPHASPRIO)
			{
				if(ddrval == DIERR_NOTINITIALIZED 
					&& ddrval == DIERR_INVALIDPARAM)
        {
          AfxMessageBox("Argh, Joystick Error : NotInitialized or InvalideParam!!!");
					return;
        }
				goto againJoystickAcquire;
			}
		}
					
		switch(pController->AxeForStickLeftRight)
		{
		case 0: stick_x = (signed char)js.lX; break;
		case 1: stick_x = (signed char)-js.lX; break;
		case 2: stick_x = (signed char)js.lY; break;
		case 3: stick_x = (signed char)-js.lY; break;
		case 4: stick_x = (signed char)js.lZ; break;
		case 5: stick_x = (signed char)-js.lZ; break;
		case -1:
			if(KEYDOWN(js.rgbButtons, pController->StickLeft))	stick_x = -64; 
			if(KEYDOWN(js.rgbButtons, pController->StickRight))	stick_x = 64; 
			break;
		}
		switch(pController->AxeForStickUpDown)
		{
		case 0: stick_y = (signed char)js.lX; break;
		case 1: stick_y = (signed char)-js.lX; break;
		case 2: stick_y = (signed char)js.lY; break;
		case 3: stick_y = (signed char)-js.lY; break;
		case 4: stick_y = (signed char)js.lZ; break;
		case 5: stick_y = (signed char)-js.lZ; break;
		case -1:
			if(KEYDOWN(js.rgbButtons, pController->StickUp))		stick_y = 64; 
			if(KEYDOWN(js.rgbButtons, pController->StickDown))	stick_y = -64; 
			break;
		}

		if(KEYDOWN(js.rgbButtons, pController->PadUp))		      button |= U_JPAD; 
		if(KEYDOWN(js.rgbButtons, pController->PadDown))		    button |= D_JPAD; 
		if(KEYDOWN(js.rgbButtons, pController->PadLeft))		    button |= L_JPAD; 
		if(KEYDOWN(js.rgbButtons, pController->PadRight))		    button |= R_JPAD;

		if(KEYDOWN(js.rgbButtons, pController->ButtonA))	      button |= A_BUTTON;
		if(KEYDOWN(js.rgbButtons, pController->ButtonB))		    button |= B_BUTTON;

		if(KEYDOWN(js.rgbButtons, pController->ButtonCUp))		  button |= U_CBUTTONS;
		if(KEYDOWN(js.rgbButtons, pController->ButtonCDown))	  button |= D_CBUTTONS;
		if(KEYDOWN(js.rgbButtons, pController->ButtonCLeft))	  button |= L_CBUTTONS;
		if(KEYDOWN(js.rgbButtons, pController->ButtonCRight))		button |= R_CBUTTONS;

		if(KEYDOWN(js.rgbButtons, pController->ButtonStart))		button |= START_BUTTON;

		if(KEYDOWN(js.rgbButtons, pController->TriggerLeft))		button |= L_TRIG;
		if(KEYDOWN(js.rgbButtons, pController->TriggerRight))   button |= R_TRIG;
		if(KEYDOWN(js.rgbButtons, pController->TriggerZ))			  button |= Z_TRIG;
	}

	pKeys->button = button;
	pKeys->X_AXIS = stick_x;
	pKeys->Y_AXIS = stick_y;

#undef KEYDOWN
}
/******************************************************************
  Function: GetKeys
  Purpose:  To get the current state of the controllers buttons.
  input:    - Controller Number (0 to 3)
            - A pointer to a BUTTONS structure to be filled with 
			the controller state.
  output:   none
*******************************************************************/  	
extern "C" __declspec(dllexport) void GetKeys(int controller_id, BUTTONS * pKeys )
{
  if(pKeys == NULL)
    return;

  Open_DI(hWndInput);
	
  iGetKeysCounter++;
  SetVoidButtons(pKeys);

  if(controller_id>=0
    && controller_id<=3)
  {
		if(g_bUseKaillera)
		{
			if(iKailleraPlayer == controller_id)
				GetButtonsFrom(&Controllers[g_iKailleraPlayAs], &KailleraBufferPlayer);
		  
			// Get next all keys
			WaitKailleraSync();
			CopyButtons(pKeys, &KailleraBufferAllPlayers[controller_id]);
		}
		else
		{
			GetButtonsFrom(&Controllers[controller_id], pKeys);
		}
  }
}

/******************************************************************
  Function: InitiateControllers
  Purpose:  This function initialises how each of the controllers 
            should be handled.
  input:    - The handle to the main window.
            - A controller structure that needs to be filled for 
			  the emulator to know how to handle each controller.
  output:   none
*******************************************************************/  
extern "C" __declspec(dllexport) void InitiateControllers (HWND hwnd, CONTROL Controls[4])
{
  Controls[0].Present = Controls[1].Present = Controls[2].Present = Controls[3].Present = TRUE;
  Controls[0].RawData = Controls[1].RawData = Controls[2].RawData = Controls[3].RawData = FALSE;
  Controls[0].Plugin = Controls[1].Plugin = Controls[2].Plugin = Controls[3].Plugin = PLUGIN_NONE;

	hWndInput = hwnd;
  if(hWndInput == NULL)
    AfxMessageBox("Hum, bad news : InitiateControllers() have hwnd = NULL!!!");
}

/******************************************************************
  Function: ReadController
  Purpose:  To process the raw data in the pif ram that is about to
            be read.
  input:    - Controller Number (0 to 3) and -1 signalling end of 
              processing the pif ram.
			- Pointer of data to be processed.
  output:   none  
  note:     This function is only needed if the DLL is allowing raw
            data.
*******************************************************************/
extern "C" __declspec(dllexport) void ReadController ( int Control, BYTE * Command )
{
  // Nothing yet
}

/******************************************************************
  Function: ControllerCommand
  Purpose:  To process the raw data that has just been sent to a 
            specific controller.
  input:    - Controller Number (0 to 3) and -1 signalling end of 
              processing the pif ram.
			- Pointer of data to be processed.
  output:   none
  
  note:     This function is only needed if the DLL is allowing raw
            data.

            the data that is being processed looks like this:
            initilize controller: 01 03 00 FF FF FF 
            read controller:      01 04 01 FF FF FF FF
*******************************************************************/
extern "C" __declspec(dllexport) void ControllerCommand ( int Control, BYTE * Command)
{
  // Not used
}
