#include "stdafx.h"
#include "noote_di.h"
#include <process.h>
#include <dinput.h>
#include "di.h"

#include "controller.h"
#include "config.h"

#include "kaillera.h"

CKailleraLog KailleraLog(NULL);

bool bKailleraDllLoaded = false;
bool bKailleraRunning = false;
int iKailleraStarted = 0;

HANDLE hKailleraThread = NULL;
HANDLE hKailleraStartEvent = NULL;
HANDLE hKailleraWaitingSyncEvent = NULL;
HANDLE hKailleraInWaitEvent = NULL;
HANDLE hKailleraSyncEvent = NULL;
DWORD dwKailleraThreadIdentifier = 0;

BUTTONS KailleraBufferPlayer;
BUTTONS KailleraBufferAllPlayers[4];
int iKailleraPlayer = -1;

int WINAPI kailleraGameCallback(char *game, int player, int maxplayers)
{
	CString strLog;

	SetVoidButtons(&KailleraBufferPlayer);
	SetVoidButtons(&KailleraBufferAllPlayers[0]);
	SetVoidButtons(&KailleraBufferAllPlayers[1]);
	SetVoidButtons(&KailleraBufferAllPlayers[2]);
	SetVoidButtons(&KailleraBufferAllPlayers[3]);

	bool bShowed = false;
	if(g_bKailleraViewLog && !KailleraLog.IsShow())
	{
		bShowed = true;
		KailleraLog.Show();
	}
	
  iKailleraPlayer = player;
	iKailleraStarted++;
	SetEvent(hKailleraStartEvent);

	// Hum, does we try to replay to a game then we don't have 'reboot' it ?
	if(iKailleraStarted>1)
	{
		if(AfxMessageBox("Argh, you try to (re)play a game without reloaded it\nThis is not suitable for rest of playing...\nI prevent this but do you want to continue ?", MB_YESNO|MB_ICONSTOP) == IDNO)
		{
			goto label_End;
		}
	}

	strLog.Format("You play to '%s' as controller%d\r\n", game, player+1);
	KailleraLog.AddLog(strLog);

  typedef struct
  {
    unsigned int c;
    BUTTONS b;
  } kbuffer;
  kbuffer buf[4];

	if(maxplayers<4)
	{
    while(iKailleraStarted && bKailleraRunning)
		{
			// Wait sync
			SetEvent(hKailleraInWaitEvent);
			WaitForSingleObject(hKailleraWaitingSyncEvent, INFINITE);
			ResetEvent(hKailleraInWaitEvent);
			
			if(!iKailleraStarted || !bKailleraRunning)
				break;
			
label_Jump:
			// Copy our master player controller
      CopyButtons(&buf[0].b, &KailleraBufferPlayer);
      buf[0].c = iGetKeysCounter;

			// Send it to kaillera
      int reclen = kailleraModifyPlayValues((void *)buf, sizeof(kbuffer));
			
			// Does we are in Time Out ?
			if(reclen == -1)
			{
        strLog.Format("Error : Kaillera return fatal TimeOut.. You're too low, man !!!\r\n");
        KailleraLog.AddLog(strLog);
				AfxMessageBox(strLog, MB_OK|MB_ICONSTOP);
				break;
			}
			// Does we have received something ?
			else if(reclen > 0)
			{
        // Verify getkey counter for each players
				for(int p=0; p<maxplayers; p++)
        {
          if(buf[p].c < iGetKeysCounter)
            goto label_Jump;
        }

				// copy received data controllers in our controller buffers
				for(p=0; p<maxplayers; p++)
				{
					CopyButtons(&KailleraBufferAllPlayers[p], &buf[p].b);
				}

        // Set sync
				SetEvent(hKailleraSyncEvent);
			}
      else
			// Else try to get some datas again.
				goto label_Jump;
		}
	}
	else
	{
		strLog.Format("You have been booted out...\r\n", game);
		KailleraLog.AddLog(strLog);
		AfxMessageBox("Sorry there're too lots of players for me : only 4 max !!!", MB_OK|MB_ICONSTOP);
	}

label_End:
	kailleraEndGame();
	strLog.Format("End of '%d' Game\r\n", game);
	KailleraLog.AddLog(strLog);

	if(bShowed)
		KailleraLog.Hide();
	
	return 0;
}

void WINAPI kailleraChatReceivedCallback(char *nick, char *text)
{
	CString strTemp;
	strTemp.Format("<%s> %s\r\n", nick, text);
	KailleraLog.AddLog(strTemp);
}

void WINAPI kailleraClientDroppedCallback(char *nick, int playernb)
{
	CString strTemp;
	strTemp.Format("Player %d as %s dropped.\r\n", playernb, nick);
	KailleraLog.AddLog(strTemp);
}

void WINAPI kailleraMoreInfosCallback(char *gamename)
{
	// Do what you want with this :)
}

unsigned long __stdcall KailleraRunning( void *dummy )
{
	kailleraInfos kInfos;

	CString strTemp;
	strTemp.LoadString(IDS_VERSION);

	kInfos.appName = (char*)(LPCTSTR)strTemp;
	kInfos.gameList = "Unknown N64 Game\0\0";
	kInfos.gameCallback = kailleraGameCallback;
	kInfos.chatReceivedCallback = kailleraChatReceivedCallback;
	kInfos.clientDroppedCallback = kailleraClientDroppedCallback;
	kInfos.moreInfosCallback = kailleraMoreInfosCallback;
	
	bKailleraRunning = true;
  kailleraSetInfos(&kInfos);
	kailleraSelectServerDialog(NULL);
	bKailleraRunning = false;

	return 1;
}

void LaunchKaillera()
{
	CloseKaillera();

  hKailleraStartEvent = CreateEvent(NULL, TRUE, FALSE, "NooTe_Di Kaillera Start Event");
	hKailleraWaitingSyncEvent = CreateEvent(NULL, TRUE, FALSE, "NooTe_Di Kaillera WaitingSync Event");
	hKailleraSyncEvent = CreateEvent(NULL, TRUE, FALSE, "NooTe_Di Kaillera Sync Event");
	hKailleraInWaitEvent = CreateEvent(NULL, FALSE, TRUE, "NooTe_Di Kaillera InWait Event");	
	hKailleraThread = CreateThread(NULL, 0, KailleraRunning, 0, 0, &dwKailleraThreadIdentifier);
		
	WaitForSingleObject(hKailleraStartEvent, INFINITE);
}

void WaitKailleraSync()
{
	WaitForSingleObject(hKailleraInWaitEvent, INFINITE);
	SetEvent(hKailleraWaitingSyncEvent);
	WaitForSingleObject(hKailleraSyncEvent, INFINITE);
}

void CloseKaillera()
{
	iKailleraStarted = 0;
	bKailleraRunning = false;

  // Hum unlock if locked
	if(hKailleraWaitingSyncEvent)
		SetEvent(hKailleraWaitingSyncEvent);
	if(hKailleraSyncEvent)
		SetEvent(hKailleraSyncEvent);

	if(hKailleraThread)
	{
		Sleep(500);
		TerminateThread(hKailleraThread, -1);
		CloseHandle(hKailleraThread);
	}
	hKailleraThread = NULL;

	if(hKailleraStartEvent)
		CloseHandle(hKailleraStartEvent);
	hKailleraStartEvent = NULL;

	if(hKailleraWaitingSyncEvent)
		CloseHandle(hKailleraWaitingSyncEvent);
	hKailleraWaitingSyncEvent = NULL;

  if(hKailleraSyncEvent)
		CloseHandle(hKailleraSyncEvent);
	hKailleraSyncEvent = NULL;

  if(hKailleraInWaitEvent)
		CloseHandle(hKailleraInWaitEvent);
	hKailleraInWaitEvent = NULL;
}

